/* 
 * Copyright (C) 2003 Star Chamber Inc.
 *
 * Sussen - sussen-backend.c
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * Author: Loren Bandiera <lorenb@starchamber.ca>
 */
 
#include "sussen.h"
#include "sussen-backend.h"
#include "sussen-fileio.h"
#include "sussen-network.h"
#include <libgnomedb/libgnomedb.h>

GdaDataModel* sussen_backend_raw (GdaConnection *conn, const gchar *buf) {
	
	GdaCommand *cmd;
	GdaDataModel *dm;

	cmd = gda_command_new (buf, GDA_COMMAND_TYPE_SQL, GDA_COMMAND_OPTION_STOP_ON_ERRORS);
	dm = gda_connection_execute_single_command (conn, cmd, NULL);

	if (!dm) {
		return NULL;
	}
	
	gda_command_free (cmd);
	return dm;
}

gint sussen_backend_sql_cmd (GdaConnection *conn, const gchar *buf) {
	
	GdaCommand *cmd;
	GList *list;
	GList *node;
	gboolean errors = FALSE;
	GdaDataModel *dm;

	cmd = gda_command_new (buf, GDA_COMMAND_TYPE_SQL, GDA_COMMAND_OPTION_STOP_ON_ERRORS);
	list = gda_connection_execute_command (conn, cmd, NULL);

	if (list != NULL) {
		for (node = g_list_first (list); node != NULL; node = g_list_next (node)) {
			dm = (GdaDataModel *) node->data;
			
			if (dm == NULL) {
				errors = TRUE;
				g_print ("error dm == NULL\n");
			} else {
				g_object_unref (dm);
			}
		}
	} else {
		
		errors = TRUE;
	}

	gda_command_free (cmd);
	return errors;
}

gint sussen_backend_non_query (GdaConnection *conn, const gchar *buf) {

	GdaCommand *cmd;
	gint number;

	cmd = gda_command_new (buf, GDA_COMMAND_TYPE_SQL, GDA_COMMAND_OPTION_STOP_ON_ERRORS);
	number = gda_connection_execute_non_query (conn, cmd, NULL);

	gda_command_free (cmd);

	return (number);
}

GdaConnection* sussen_backend_init (GdaClient *client, gboolean first_time) {

	GdaConnection *conn;
	gchar *gdb_str = get_backend_xml ();
	gchar **split = g_strsplit (gdb_str,":",3);

	if (split) {	
		conn = gda_client_open_connection (client, split[0], split[1], split[2], 0);
		g_strfreev (split);
		g_free (gdb_str);
	} else {
		return NULL;
	}
	
	if (conn) {
		
		if (first_time) {
			
			sussen_backend_non_query (conn, "CREATE TABLE sessions ( \
				session_id VARCHAR(50) NOT NULL, \
				start_time DATETIME, \
				end_time DATETIME, \
				PRIMARY KEY (session_id));");
			
			sussen_backend_non_query (conn, "CREATE TABLE scan_data ( \
					record_id INTEGER NOT NULL AUTO_INCREMENT, \
					session_id VARCHAR(50) NOT NULL, \
					ip_address VARCHAR(128), \
					subnet VARCHAR(128), \
					service VARCHAR(48), \
					type INTEGER, \
					plugin_id INTEGER, \
					cve_id VARCHAR(32), \
					risk_level VARCHAR(32), \
					description BLOB, \
					acceptable BOOL, \
					false_positive BOOL, \
					is_fixed BOOL, \
					user_comments BLOB, \
					PRIMARY KEY (record_id));");
			
			return conn;
			
		} else {
			return conn;
		}
	} else {
		return NULL;
	}
}

void sussen_backend_insert_data (gchar *msg, gint type, const gchar *session_name) {
	
	gint sussen_backend_type = 0;
	gchar *sql;
	gchar *msgt = NULL;
	
	switch (type) {
		
		case MSG_NOTE:
			msgt = "Security Note";
			sussen_backend_type = 1;
			break;

   		case MSG_HOLE:
			msgt = "Security Hole";
			sussen_backend_type = 3;
			break;
   
		case MSG_INFO:
			msgt = "Security Warning";
			sussen_backend_type = 2;
			break;

		default:
			g_print ("unknown message == [%s]\n", msg);
			break;
		
	}
						
	gchar *hostname = g_strstrip(strtok (NULL, "<|>"));
	gchar *port = g_strstrip(strtok (NULL, "<|>"));
	gchar *data = g_strstrip(strtok (NULL, "<|>"));
	gchar *script_id = g_strstrip(strtok (NULL, "<|>"));
					
	if (!hostname || !port || !data || !script_id) {
		return;
	}
					
	/* FIXME: cheezy way to make subnet */
	gchar *cp2 = g_strdup(hostname);
	gchar *s1 = strtok (cp2, ".");
	gchar *s2 = strtok (NULL, ".");
	gchar *s3 = strtok (NULL, ".");
	gchar *subnet = g_strdup_printf ("%s.%s.%s", s1, s2, s3);					
					
	sql = g_strdup_printf ("INSERT INTO scan_data (session_id,ip_address,subnet,service,type,plugin_id,cve_id, risk_level,description,acceptable,false_positive,is_fixed,user_comments) VALUES ('%s','%s','%s','%s',%d, %d,'CVE','%s',\"%s\",0,0,0,'')",
							session_name, hostname, subnet, port, 
							sussen_backend_type, atoi(script_id),
							msgt,data);
	sussen_backend_non_query (sussen_be_connection, sql);

	g_free (subnet);
	g_free (sql);	

}

void sussen_backend_insert_port (gchar *msg, const gchar *session_name) {

	gchar *sql;
	
	strtok (msg, "<|>");
	strtok (NULL, "<|>"); 
	gchar *hostname = g_strstrip(strtok (NULL, "<|>"));
	gchar *port = g_strstrip(strtok (NULL, "<|>"));
	
	/* FIXME: cheezy way to make subnet */
	gchar *cp2 = g_strdup(hostname);
	gchar *s1 = strtok (cp2, ".");
	gchar *s2 = strtok (NULL, ".");
	gchar *s3 = strtok (NULL, ".");
	gchar *subnet = g_strdup_printf ("%s.%s.%s", s1, s2, s3);					
	
	sql = g_strdup_printf ("INSERT INTO scan_data (session_id,ip_address,subnet,service,type,plugin_id,cve_id, risk_level,description,acceptable,false_positive,is_fixed,user_comments) VALUES ('%s','%s','%s','%s',0,0,'CVE','','',0,0,0,'')", 
 								session_name, hostname, subnet, port);
	sussen_backend_non_query (sussen_be_connection, sql);
	g_free (sql);
}
