/* 
 * Copyright (C) 2003 Star Chamber Inc.
 *
 * Sussen - sussen-network.c
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * Author: Loren Bandiera <lorenb@starchamber.ca>
 */

#include "sussen.h"
#include "sussen-backend.h"
#include "sussen-cert-druid.h"
#include "sussen-fileio.h"
#include "sussen-network.h"
#include "sussen-report.h"
#include "sussen-report-druid.h"
#include "sussen-util.h"
#include <stdarg.h>
#include <glib.h>
#include <gnet.h>

gchar *attack_plugin_list = NULL;
GHashTable *scan_hosts;
gboolean scan_running;
gboolean restore_session;
GHashTable *scanning_current_hosts; 

gboolean sussen_network_write (GTcpSocket *socket, gchar *str) {
	
	GIOChannel *gio;
	GIOError err;
	gsize bytes_written;
	
	gio = gnet_tcp_socket_get_io_channel (socket);
	err = gnet_io_channel_writen (gio, str, strlen(str), &bytes_written);
	if (err) {
		g_print ("error trying to send [%s] to server\n", str);
		return FALSE;
	}
	
	return TRUE;	
}

gchar* sussen_network_read (GTcpSocket *socket, gint length) {
	
	GIOChannel *gio;
	GIOError err;
	gsize bytes_read;
	
	gchar *buf = g_malloc (length);
	
	gio = gnet_tcp_socket_get_io_channel (socket);
	err = gnet_io_channel_readline (gio, buf, length, &bytes_read);
	if (err) {
		g_print ("error trying to read from server\n");
		g_free (buf);
		return NULL;
	}
	
	return buf;
}

static GData* sussen_network_parse_plugins (gchar *buf) {
	
	gchar *id;
	gchar *name;
	gchar *category;
	gchar *copyright;
	gchar *description;
	gchar *summary;
	gchar *family;
	gchar *version;
	gchar *cve_id;
	gchar *bugtraq_id;
	
	gchar *cp = g_strdup (buf);
	id = g_strstrip(strtok (cp, "<|>"));  
	name = g_strstrip(strtok (NULL, "<|>"));
	category = g_strstrip(strtok (NULL, "<|>"));
	copyright = g_strstrip(strtok (NULL, "<|>"));	
	description = g_strstrip(strtok (NULL, "<|>"));
	summary = g_strstrip(strtok (NULL, "<|>"));
	family = g_strstrip(strtok (NULL, "<|>"));
	version = strtok (NULL, "<|>");
	cve_id = strtok (NULL, "<|>");
	bugtraq_id = strtok (NULL, "<|>");
	
	GData *plugin_datalist;	
	g_datalist_init (&plugin_datalist);
	
	g_datalist_set_data (&plugin_datalist, "ID", id);
	g_datalist_set_data (&plugin_datalist, "ASC_ID", id);	
	g_datalist_set_data (&plugin_datalist, "NAME", name);
	g_datalist_set_data (&plugin_datalist, "CATEGORY", category);
	g_datalist_set_data (&plugin_datalist, "COPYRIGHT", copyright);
	g_datalist_set_data (&plugin_datalist, "DESCRIPTION", description);
	g_datalist_set_data (&plugin_datalist, "SUMMARY", summary);
	g_datalist_set_data (&plugin_datalist, "FAMILY", family);

	gchar *test = strchr (version,'$');
	if (test) {
		g_datalist_set_data (&plugin_datalist, "VERSION", g_strstrip(version));
	}
	
	if (cve_id) {
	
		if (g_ascii_strncasecmp(g_strstrip(cve_id),"CAN",3) == 0 || 
			g_ascii_strncasecmp(g_strstrip(cve_id),"CVE",3) == 0) {
				
				g_datalist_set_data (&plugin_datalist, "CVE_ID", g_strstrip(cve_id));
		}
	}
	
	if (bugtraq_id) {
		
		gint test = atoi (g_strstrip(bugtraq_id));
		if (test) {
			g_datalist_set_data (&plugin_datalist, "BUGTRAQ_ID", g_strstrip(bugtraq_id));
		}
	}

	if ((!g_ascii_strcasecmp (category, "denial")) ||
		(!g_ascii_strcasecmp (category, "kill_host")) ||
		(!g_ascii_strcasecmp (category, "destructive_attack"))) {			
			g_datalist_set_data (&plugin_datalist, "ENABLED", "0");	
	} else {
		if (!g_ascii_strcasecmp (category, "scanner")) {			
			if (!g_ascii_strcasecmp (name, "Nmap")) {
				g_datalist_set_data (&plugin_datalist, "ENABLED", "1");	
			} else {
				g_datalist_set_data (&plugin_datalist, "ENABLED", "0");	
			}
		} else {
			g_datalist_set_data (&plugin_datalist, "ENABLED", "1");	
		}		
	}
		
	return plugin_datalist;	
}

void sussen_network_connect (const gchar *hostname, gint port, const gchar *login, const gchar *pass) {

	GTcpSocket *socket;
	gchar *str;
	gchar *buf;
	
	socket = gnet_tcp_socket_connect (hostname, port);
	if (!socket) {
		sussen_error_dialog (_("Unable to connect to server\n"));
		return;
	}
	global_gtcpsocket = socket;	
	
	sussen_network_write (socket, NESSUS_PROTO_NAME);
	buf = sussen_network_read (socket, 14);
	
	gchar *new_buf = g_strdup(g_strchomp (buf));
	g_free (buf);
			
	if (g_ascii_strcasecmp(new_buf, "< NTP/1.2 >") == 0) {

		buf = sussen_network_read (socket, 7);

		if (g_ascii_strcasecmp(buf, "User :") == 0) {
		
			g_free (buf);
			str = g_strdup_printf ("%s\n", login);	
			sussen_network_write (socket, str);
			g_free (str);
			
			buf = sussen_network_read (socket, 12);
				
			if (g_ascii_strcasecmp (g_strstrip(buf), "Password :") != 0) {
				g_print ("didn't get response to User command\n");
				return;
			} 

			str = g_strdup_printf ("%s\n", pass);	
			sussen_network_write (socket, str);
			g_free (str);
			
			/* get plugin list from server */
				
			g_free (buf);			
			buf = sussen_network_read (socket, 29);
			if (g_ascii_strcasecmp (g_strstrip(buf), "SERVER <|> PLUGIN_LIST <|>") == 0) {
				
				g_free (buf);
				gboolean done = FALSE;
				
				while (!done) {

					buf = sussen_network_read (socket, 32768);
					if (!buf) {
						g_print ("error: reading plugin list\n");
						done = TRUE;
					}
					
					if(!strncmp(buf, "<|> SERVER", 10)) {
						done = TRUE;
					} else {
						GData *plug = sussen_network_parse_plugins (g_strstrip(buf));	
						plugin_list = g_slist_append (plugin_list, (gpointer)plug);
					}
					
					g_free (buf);
				}

				/* get nessusd preferences */
			
				buf = sussen_network_read (socket, 32768);
				if (g_ascii_strncasecmp (g_strstrip(buf), "SERVER <|> PREFERENCES <|>", 26) == 0) {
				
					g_free (buf);
					done = FALSE;
					while (!done) {

						buf = sussen_network_read (socket, 32768);
						if (!buf) {
							g_print ("error: reading plugin list\n");
							done = TRUE;
						}

						if(!g_ascii_strncasecmp (buf, "<|> SERVER", 10)) {
							done = TRUE;
						} else {
							
							gchar *test;
							
							test = strchr (buf, ':');
							if (test) {
								/* FIXME: plugin preferences 
								 *
								 * example: 
								 *
								 *	SMTP settings[entry]:Third party domain : <|> nessus.org
								 *  plugin name[widget type]:name <|> value
								 */
								
							} else {							
								
								if (g_ascii_strncasecmp (g_strstrip(buf), "ntp_save_sessions", 17) == 0) {								
									
									gchar *cp = g_strdup (g_strstrip(buf));
									gchar *pref = g_strstrip(strtok (cp, "<|>"));
									gchar *value = g_strstrip(strtok (NULL, "<|>"));
																										
									if (g_ascii_strcasecmp (value, "yes") == 0) {
										sussen_save_sessions = TRUE;
										pref = NULL;
									} 
									
								} else if (g_ascii_strncasecmp (g_strstrip(buf), "ntp_detached_sessions", 21) == 0) {

									gchar *cp = g_strdup (g_strstrip(buf));
									gchar *pref = g_strstrip(strtok (cp, "<|>"));
									gchar *value = g_strstrip(strtok (NULL, "<|>"));
																										
									if (g_ascii_strcasecmp (value, "yes") == 0) {
										sussen_save_detached_sessions = TRUE;
										pref = NULL;
									} 									

								} else if (g_ascii_strncasecmp (g_strstrip(buf), "server_info_", 12) == 0) {

									gchar *cp = g_strdup (g_strstrip(buf));
									gchar *pref = g_strstrip(strtok (cp, "<|>"));
									gchar *value = g_strstrip(strtok (NULL, "<|>"));
							
									g_datalist_set_data (&server_information, pref, value);

								} else {

									gchar *cp = g_strdup (g_strstrip(buf));
									gchar *pref = g_strstrip(strtok (cp, "<|>"));
									gchar *value = g_strstrip(strtok (NULL, "<|>"));

									g_datalist_set_data (&server_preferences, pref, value);
								}
							}
						}
						
						g_free (buf);
					}
				}

				/* get server rules */
				
				buf = sussen_network_read (socket, 32768);				
				if (g_ascii_strncasecmp (g_strstrip(buf), "SERVER <|> RULES <|>", 20) == 0) {				
					
					g_free (buf);
					done = FALSE;
					while (!done) {

						buf = sussen_network_read (socket, 32768);
						if (!buf) {
							g_print ("error: reading rule list\n");
							done = TRUE;
						}

						if(!g_ascii_strncasecmp (buf, "<|> SERVER", 10)) {
							done = TRUE;
						} else {
							server_rules = g_slist_append (server_rules, g_strdup(g_strstrip(buf)));
						}
						
						g_free (buf);
					}
					
				}

				/* get plugin dependencies */
				
				buf = sussen_network_read (socket, 32768);
				if (g_ascii_strcasecmp (g_strstrip(buf), "SERVER <|> PLUGINS_DEPENDENCIES") == 0) {				
					
					g_free (buf);
					done = FALSE;
					while (!done) {

						buf = sussen_network_read (socket, 32768);
						if (!buf) {
							g_print ("error: reading dependencies\n");
							done = TRUE;
						}

						if(!g_ascii_strncasecmp (buf, "<|> SERVER", 10)) {
							done = TRUE;
						} else {
							gchar *cp = g_strdup (g_strstrip(buf));
							gchar *plug = g_strstrip(strtok (cp, "<|>"));
							gchar *depends = g_strstrip(strtok (NULL, "<|>"));
							
							g_datalist_set_data (&plugin_depends, plug, depends);							
						}
						
						g_free (buf);
					}
					
				}

				/* get session list */
				str = g_strdup_printf ("CLIENT <|> SESSIONS_LIST <|> CLIENT\n");
				sussen_network_write (socket, str);
				g_free (str);			
				
				buf = sussen_network_read (socket, 32768);
				if (g_ascii_strcasecmp (g_strstrip(buf), "SERVER <|> SESSIONS_LIST") == 0) {

					g_free (buf);
					done = FALSE;
					while (!done) {

						buf = sussen_network_read (socket, 32768);
						if (!buf) {
							g_print ("error: reading sessions\n");
							g_print ("read from server: [%s]\n", buf);
							done = TRUE;
						}

						if(!g_ascii_strncasecmp (buf, "<|> SERVER", 10)) {
							done = TRUE;
						} else {
							
							gchar *cp = g_strdup (g_strstrip(buf));
							gchar **split = g_strsplit (cp," ",2);
					
							g_hash_table_insert (server_sessions, g_strdup(split[0]), g_strdup(split[1]));
							g_strfreev (split);
						}
						
						g_free (buf);
					}						
				}					
				
				gchar *status_msg = g_strdup_printf ("Connected to '%s'", hostname);
				update_status (status_msg);
				g_free (status_msg);

				update_connected_status (hostname, TRUE);
				set_target_defaults ();				
				set_widgets_sensitive (TRUE,FALSE);
			
				GtkWidget *w;
	
				w = glade_xml_get_widget (mainwindow_xml,"connect_button");
				gtk_widget_set_sensitive (w, FALSE);
				w = glade_xml_get_widget (mainwindow_xml,"connect1");
				gtk_widget_set_sensitive (w, FALSE);	
				w = glade_xml_get_widget (mainwindow_xml,"stop_scan_button");
				gtk_widget_set_sensitive (w, FALSE);
				w = glade_xml_get_widget (mainwindow_xml,"stop_scan1");
				gtk_widget_set_sensitive (w, FALSE);	
				w = glade_xml_get_widget (mainwindow_xml,"stop_scanning_host_button");
				gtk_widget_set_sensitive (w, FALSE);
				
			} else {
				
				gchar *str = g_strdup_printf ("Invalid Login\n");
				sussen_error_dialog (str);
				g_free (str);
				
				return;
			}
			
		} else {
			g_print ("didn't get response to NTP command\n");
		}
		
		
	} else {
		g_print ("invalid protocol\n");
	}
	
	g_free (buf);
}

void sussen_network_disconnect (void) {
	
	gnet_tcp_socket_delete  (global_gtcpsocket);
	global_gtcpsocket = NULL;
	update_status ("Logged out");
}

void send_server_preference (GQuark key_id, gpointer data, gpointer user_data) {
	
	GTcpSocket *socket = global_gtcpsocket;
	gchar *name;
	gchar *value;

	name = (gchar *)g_quark_to_string (key_id);
	value = (gchar *) data;
	
	gchar *str = g_strdup_printf ("%s <|> %s\n", name, value);
	sussen_network_write (socket, str);
	g_free (str);
}

void build_plugin_list (gpointer data, gpointer user_data) {
	
	GData *cur_plugin = (GData *)data;
	gchar *enabled;
	gchar *id;
	gint value;
		
	id = g_datalist_get_data (&cur_plugin, "ID");
	enabled = g_datalist_get_data (&cur_plugin, "ENABLED");	
	value = atoi (enabled);
	
	if (use_policy_file == 1) {
		gchar *test = g_hash_table_lookup (plugin_hash,id);
		if (test) {
			strcat (attack_plugin_list, id);
			strcat (attack_plugin_list, ";");		
		}		
	} else {	
		if (value == 1) {		
			gchar *id = g_datalist_get_data (&cur_plugin, "ID");
			strcat (attack_plugin_list, id);
			strcat (attack_plugin_list, ";");
		}
	}
}

gint sussen_network_parse_msg (gchar *msg, const gchar *session_name) {

	gint type = 0;
	gchar *sql;
	
	if (g_ascii_strncasecmp (msg, "s:", 2) == 0) {
		return MSG_STAT2;
	} else {
		
		if (g_ascii_strncasecmp (msg, "SERVER", 6) == 0) {

			gchar *cp = g_strdup (msg);
			gchar *srv = g_strstrip(strtok (cp, "<|>"));
			gchar *str_type = g_strstrip(strtok (NULL, "<|>"));
			srv = NULL;
						
			if (!g_ascii_strcasecmp(MSG_HOLE_STR, str_type))
				type = MSG_HOLE;
			
			if (!g_ascii_strcasecmp(MSG_INFO_STR, str_type))
				type = MSG_INFO;
			
			if (!g_ascii_strcasecmp(MSG_NOTE_STR, str_type))
				type = MSG_NOTE;
			
			if (!g_ascii_strcasecmp(MSG_TIME_STR, str_type))
				type = MSG_TIME;

			if (!g_ascii_strcasecmp(MSG_STAT_STR, str_type))
				type = MSG_STAT;

			if (!g_ascii_strcasecmp(MSG_PORT_STR, str_type))
				type = MSG_PORT;

			if (!g_ascii_strcasecmp(MSG_ERROR_STR, str_type))
				type = MSG_ERROR;

			if (!g_ascii_strcasecmp(MSG_PING_STR, str_type))
				type = MSG_PING;

			if (!g_ascii_strcasecmp(MSG_PLUGINS_ORDER_STR, str_type))
				type = MSG_PLUGINS_ORDER;

			if (!g_ascii_strcasecmp(MSG_FINISHED_STR, str_type))
				type = MSG_FINISHED;
			
			if (!g_ascii_strcasecmp(MSG_BYE_STR, str_type))
				type = MSG_BYE;
			
			switch (type) {
  
				case MSG_TIME: {
					
					gint type = 0;
					gchar *new_msg = g_strstrip(strtok (NULL, "<|>"));

					if (g_ascii_strcasecmp (new_msg, "SCAN_START") == 0){
						type = TIMER_SCAN_START;
					} else if(g_ascii_strcasecmp (new_msg, "SCAN_END") == 0) {
						type = TIMER_SCAN_END;
					}

					if (type) {

						/* FIXME: need to reformat date to YYYY-MM-DD HH:MM:SS */						
						gchar *date = g_strstrip(strtok (NULL, "<|>"));
						
						switch (type) {
							
							case TIMER_SCAN_START: {

								sql = g_strdup_printf ("UPDATE sessions SET start_time='%s' WHERE session_id='%s'", date, session_name);
								sussen_backend_non_query (sussen_be_connection, sql);								
							  	g_free (sql);								
								break;
							}
							
							case TIMER_SCAN_END: {

								sql = g_strdup_printf ("UPDATE sessions SET end_time='%s' WHERE session_id='%s'", date, session_name);
								sussen_backend_non_query (sussen_be_connection, sql);								
							  	g_free (sql);
								break;
							}
							
						}
					}
					break;
				}
				
				case MSG_STAT2: {
					return (MSG_STAT2);
					break;
				}

				case MSG_ERROR: {
					break;
				}
				
				case MSG_PORT: {
					sussen_backend_insert_port (msg, session_name);
					break;
				}

				case MSG_HOLE: {
					sussen_backend_insert_data (msg, MSG_HOLE, session_name);
					break;
				}

				case MSG_INFO: {
					sussen_backend_insert_data (msg, MSG_INFO, session_name);
					break;
				}

				case MSG_NOTE: {
					sussen_backend_insert_data (msg, MSG_NOTE, session_name);					
					break;
				}

				case MSG_STAT: {
					return(MSG_STAT);
					break;
				}

				case MSG_FINISHED: {				
					break;
				}
				
				case MSG_PING: {
					
					gchar *str = g_strdup_printf ("CLIENT <|> PONG <|> PING <|> CLIENT\n\n");
					sussen_network_write (global_gtcpsocket, str);
					g_free (str);
					return MSG_PING;
					break;
				}

				case MSG_PLUGINS_ORDER: {
					return (MSG_PLUGINS_ORDER);
					break;
				}

				case MSG_BYE: {
					return (MSG_BYE);
					break;
				}				
			}
			
			return -1;
			
		} else {
			return -1;
		}
	}
}

gboolean update_each_attack (GtkTreeModel *model, GtkTreePath *path, GtkTreeIter *iter, GHashTable *hosts) {
	
	gchar *cur_host;
		
	if (model && iter) {

		gtk_tree_model_get (GTK_TREE_MODEL(scan_status_store),iter,0,&cur_host,-1);
		if (strcmp(cur_host,g_hash_table_lookup (hosts,"hostname")) == 0) {
			gtk_list_store_set (GTK_LIST_STORE(scan_status_store), iter,
								2, g_strdup(g_hash_table_lookup (hosts,"percent")),
								-1);
		} 
		
		g_free (cur_host);
		return FALSE;
	}
	
	return TRUE;
}


gboolean update_each_portscan (GtkTreeModel *model, GtkTreePath *path, GtkTreeIter *iter, GHashTable *hosts) {

	gchar *cur_host;
		
	if (model && iter) {

		gtk_tree_model_get (GTK_TREE_MODEL(scan_status_store),iter,0,&cur_host,-1);
		if (strcmp(cur_host,g_hash_table_lookup (hosts,"hostname")) == 0) {
			gtk_list_store_set (GTK_LIST_STORE(scan_status_store), iter,
									1, g_strdup(g_hash_table_lookup (hosts,"percent")),
									-1);
		} 
		
		g_free (cur_host);
		return FALSE;
	}
	
	return TRUE;
}

void update_scan_status (gchar *msg, gint short_status) {

	gchar *hostname = NULL;
	gchar *action = NULL;
	gchar *current = NULL;
	gint max = 0;
	gfloat gmax;
	gfloat gcurrent;
	
	if (!short_status) {
		g_print ("msg == [%s]\n", msg);
	} else {
		gchar *cp = g_strstrip(g_strdup (msg));
		strtok (cp, ":");
		gchar *check_action = strtok (NULL, ":");
		if (g_ascii_strncasecmp (check_action, "p", 1) == 0) {
			action = g_strdup ("portscan");
		} else {
			action = g_strdup ("attack");
		}
		
		hostname = strtok (NULL, ":");
		current = strtok (NULL, ":");
		max = atoi (strtok (NULL, ":"));
	}
	
	if (hostname) {

		if (g_hash_table_lookup (scanning_current_hosts, (gconstpointer) hostname) != NULL) {
		
			gfloat f;

			gmax = max;
			gcurrent = atoi(current);
			f = (gcurrent/gmax);

			if(f>1)
				f=1;
			if(f<0.0)
				f=0.0;		
		
			gint test;
			test = f * 100;
			gchar *test2 = g_strdup_printf ("%d%%", test);
			g_hash_table_insert (scan_hosts,"hostname",hostname);
			g_hash_table_insert (scan_hosts,"percent", test2);
			
			if( strcmp(action, "portscan") == 0 ) {
				gtk_tree_model_foreach (GTK_TREE_MODEL(scan_status_store), (void *)update_each_portscan,
										scan_hosts);
			} else {
				gtk_tree_model_foreach (GTK_TREE_MODEL(scan_status_store), (void *)update_each_attack,
										scan_hosts);
			}
			g_free (test2);				
		} else {

			g_hash_table_insert (scanning_current_hosts, hostname, "yes");
			
			GtkTreeIter iter;
			
			gtk_list_store_append (scan_status_store,&iter);
			gtk_list_store_set (GTK_LIST_STORE(scan_status_store), &iter,
								0, g_strdup(hostname),
								-1);
			
		}
		
	}
}

gboolean io_read_callback (GIOChannel *source, GIOCondition condition, gpointer data) {
	
	gchar *buf;
	gboolean done = FALSE;
	gint type = 0;
	gint finished = 0;
	gchar *session_name = (gchar*)data;
	
	if (!global_gtcpsocket) {
		sussen_error_dialog ("connection to server lost\n");
		done = TRUE;
	}

	while (!done) {
		
		buf = sussen_network_read (global_gtcpsocket, 32678);
		if (!buf) {
			done = TRUE;
		} else {
			type = sussen_network_parse_msg (g_strstrip(buf), g_strdup(session_name));

			switch(type) {
				case MSG_BYE: {
					gchar *str = g_strdup_printf ("CLIENT <|> BYE <|> ACK\n");
					sussen_network_write (global_gtcpsocket, str);
					g_free (str);
					finished = 1;
					break;
				}
		
				case MSG_STAT2 : 
					update_scan_status (buf, 1);
					break;
	
				case MSG_STAT :
					update_scan_status (buf, 0);
					break;

				case MSG_PLUGINS_ORDER :
					/* FIXME: haven't yet gotten this message from the server; not sure
					   what to do with it */
					break;

				case MSG_FINISHED :				
					break;	
			}
			
			if (finished) {
				F_nessusd_running = 0;
				scan_running = FALSE;

				if (restore_session)
					restore_session = FALSE;
		
				set_widgets_sensitive (TRUE,FALSE);
				GtkWidget *w = glade_xml_get_widget (mainwindow_xml, "stop_scan_button");
				gtk_widget_set_sensitive (w,FALSE);
				w = glade_xml_get_widget (mainwindow_xml, "stop_scan1");
				gtk_widget_set_sensitive (w,FALSE);
		
				gint user_response;
				user_response = show_report_druid (g_strdup(session_name));
		
				switch (user_response) {
			
					case REPORT_DRUID_VIEW:
					case REPORT_DRUID_VIEW_EXPORT:
						show_results (g_strdup(session_name));
						break;
			
					case REPORT_DRUID_EXPORT:
						break;
			
					case REPORT_DRUID_CANCEL:
						break;
				
					default:
						break;
				}			
			} 
			
			g_free (buf);
			return TRUE;
		}
	}
	
	return FALSE;
}

void sussen_network_send_preferences (void) {
	
	GTcpSocket *socket = global_gtcpsocket;
	gchar *str;
	gchar *buf;

	attack_plugin_list = g_malloc (2000*50+1+5*50+1);
	g_slist_foreach (plugin_list, build_plugin_list, NULL);
		
	/* FIXME: manually adding two scanners to the plugin_list */
	strcat (attack_plugin_list, "10180;10335;");
		
	scanning_current_hosts = g_hash_table_new (g_str_hash, g_str_equal);
	scan_hosts = g_hash_table_new (g_str_hash, g_str_equal);
	
	/* send preferences */
	
	str = g_strdup_printf ("CLIENT <|> PREFERENCES <|>\n");
	sussen_network_write (socket, str);
	g_free (str);
	
	str = g_strdup_printf ("ntp_opt_show_end <|> yes\n");
	sussen_network_write (socket, str);
	g_free (str);

	str = g_strdup_printf ("ntp_keep_communication_alive <|> yes\n");
	sussen_network_write (socket, str);
	g_free (str);

	str = g_strdup_printf ("ntp_short_status <|> yes\n");
	sussen_network_write (socket, str);
	g_free (str);

	str = g_strdup_printf ("ntp_client_accepts_notes <|> yes\n");
	sussen_network_write (socket, str);
	g_free (str);

	str = g_strdup_printf ("ntp_escape_crlf <|> yes\n");
	sussen_network_write (socket, str);
	g_free (str);

	g_datalist_set_data (&server_preferences, "plugin_set", g_strdup(attack_plugin_list));
	g_datalist_foreach (&server_preferences, send_server_preference, NULL);	
	
	/* FIXME: send plugin preferences 
	 *
	 * example
	 *	%s[%s]:%s <|> %s\n
	 */

	str = g_strdup_printf ("<|> CLIENT\n");
	sussen_network_write (socket, str);
	g_free (str);
	
	/* get_preferences_errors */
	buf = sussen_network_read (socket, 512);
	g_free (buf);
	
	buf = sussen_network_read (socket, 512);
	g_free (buf);	
}

void sussen_network_attack (const gchar *hostname, const gchar *session_name) {

	GTcpSocket *socket = global_gtcpsocket;
	gchar *str;

	sussen_network_send_preferences ();	
	/* FIXME: set rules */
	
	/* Send attack command */
	str = g_strdup_printf ("CLIENT <|> LONG_ATTACK <|>\n");
	sussen_network_write (socket, str);
	g_free (str);

	str = g_strdup_printf ("%d\n", strlen(hostname));
	sussen_network_write (socket, str);
	g_free (str);

	str = g_strdup_printf ("%s\n", hostname);
	sussen_network_write (socket, str);
	g_free (str);
	
	GIOChannel *gio = gnet_tcp_socket_get_io_channel (socket);
	g_io_add_watch (gio, G_IO_IN, io_read_callback, (gpointer)session_name);

	g_free (attack_plugin_list);
}

void sussen_network_restore_session (const gchar *session_name) {
	
	GTcpSocket *socket = global_gtcpsocket;
	gchar *str;

	sussen_network_send_preferences ();	
	/* FIXME: set rules */

	str = g_strdup_printf ("CLIENT <|> SESSION_RESTORE <|> %s <|> CLIENT\n", session_name);
	sussen_network_write (socket, str);
	g_free (str);
	
	GIOChannel *gio = gnet_tcp_socket_get_io_channel (socket);
	g_io_add_watch (gio, G_IO_IN, io_read_callback, (gpointer)session_name);
	
	g_free (attack_plugin_list);
}
